@@ -5,9 +5,10 @@ package tproxy
 
 import (
 	"errors"
-	"github.com/tobyxdd/hysteria/pkg/core"
 	"net"
 	"time"
+
+	"github.com/tobyxdd/hysteria/pkg/core"
 )
 
 var ErrTimeout = errors.New("inactivity timeout")
@@ -15,7 +16,8 @@ var ErrTimeout = errors.New("inactivity timeout")
 type UDPTProxy struct{}
 
 func NewUDPTProxy(hyClient *core.Client, listen string, timeout time.Duration,
-	connFunc func(addr, reqAddr net.Addr), errorFunc func(addr, reqAddr net.Addr, err error)) (*UDPTProxy, error) {
+	connFunc func(addr, reqAddr net.Addr), errorFunc func(addr, reqAddr net.Addr, err error),
+) (*UDPTProxy, error) {
 	return nil, errors.New("not supported on the current system")
 }
 
